#include "BLE.h"

/* std C lib */
#include <malloc.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

/* Cypress */
#include "cybsp.h"
#include "cyhal.h"

/* WICED */
#include "cybt_platform_config.h"
#include "cycfg_bt_settings.h"
#include "cycfg_gap.h"
#include "cycfg_gatt_db.h"
#include "wiced_bt_gatt.h"
#include "wiced_bt_stack.h"

int isConnected = 0;
int isAdvertising = 0;

void BLE_Init() {
    wiced_result_t wStatus;

    cybt_platform_config_init(&cybsp_bt_platform_cfg);

    wStatus = wiced_bt_stack_init(BLE_ManagementCallback, &wiced_bt_cfg_settings);
    CY_ASSERT(wStatus == WICED_BT_SUCCESS);
}

wiced_bt_gatt_status_t BLE_GattCallback(wiced_bt_gatt_evt_t event, wiced_bt_gatt_event_data_t* eventData) {
    printf("GattCallback(%d) => ", event);

    wiced_bt_gatt_status_t wStatus;

    if (event == GATT_ATTRIBUTE_REQUEST_EVT) {
        uint16_t connId = eventData->attribute_request.conn_id;
        uint16_t opcode = eventData->attribute_request.opcode;
        uint16_t len = eventData->attribute_request.len_requested;
        if (eventData->attribute_request.opcode == GATT_REQ_READ) {
            wiced_bt_gatt_read_t* req = &eventData->attribute_request.data.read_req;
            wStatus = BLE_GattRead(connId, opcode, req, len);
            printf("%d\r\n", wStatus);
            return wStatus;
        } else if (eventData->attribute_request.opcode == GATT_REQ_WRITE) {
            wStatus = BLE_GattWrite(connId, opcode, &eventData->attribute_request.data.write_req);
            printf("%d\r\n", wStatus);
            return wStatus;
        } else if (eventData->attribute_request.opcode == GATT_REQ_MTU) {
            uint16_t mtu = eventData->attribute_request.data.remote_mtu;
            wStatus = wiced_bt_gatt_server_send_mtu_rsp(connId, mtu, wiced_bt_cfg_settings.p_ble_cfg->ble_max_rx_pdu_size);
            printf("%d\r\n", wStatus);
            return wStatus;
        } else if (eventData->attribute_request.opcode == GATT_REQ_READ_BY_TYPE) {
            wiced_bt_gatt_read_by_type_t* req = &eventData->attribute_request.data.read_by_type;

            uint8_t* buffer = malloc(len);
            if (buffer == NULL) {
                wiced_bt_gatt_server_send_error_rsp(connId, opcode, req->s_handle, WICED_BT_GATT_INSUF_RESOURCE);
                return WICED_BT_GATT_INSUF_RESOURCE;
            }

            uint8_t* wrPtr = buffer;
            uint16_t bufferSize = len;
            uint8_t pair_len = 0;
            for (uint16_t h = req->s_handle; h < req->e_handle; h++) {
                uint16_t handle = wiced_bt_gatt_find_handle_by_type(h, req->e_handle, &req->uuid);

                for (int i = 0; i < app_gatt_db_ext_attr_tbl_size; i++) {
                    if (app_gatt_db_ext_attr_tbl[i].handle == handle) {
                        int written = wiced_bt_gatt_put_read_by_type_rsp_in_stream(wrPtr, bufferSize, &pair_len, h, app_gatt_db_ext_attr_tbl[i].cur_len, app_gatt_db_ext_attr_tbl[i].p_data);
                        wrPtr += written;
                        bufferSize -= written;
                    }
                }
            }

            wStatus = wiced_bt_gatt_server_send_read_by_type_rsp(connId, opcode, pair_len, wrPtr - buffer, buffer, (void*)free);

            printf("%d\r\n", wStatus);
            return wStatus;
        } else {
            printf("xx %d\r\n", eventData->attribute_request.opcode);
            printf("%d\r\n", WICED_BT_GATT_ERROR);
            return WICED_BT_GATT_ERROR;
        }
    } else if (event == GATT_GET_RESPONSE_BUFFER_EVT) {
        eventData->buffer_request.buffer.p_app_rsp_buffer = (uint8_t*)malloc(eventData->buffer_request.len_requested);
        eventData->buffer_request.buffer.p_app_ctxt = (void*)free;
        printf("%d\r\n", WICED_BT_GATT_SUCCESS);
        return WICED_BT_GATT_SUCCESS;
    } else if (event == GATT_APP_BUFFER_TRANSMITTED_EVT) {
        if (eventData->buffer_xmitted.p_app_ctxt == free) {
            free(eventData->buffer_xmitted.p_app_data);
        }
        printf("%d\r\n", WICED_BT_GATT_SUCCESS);
        return WICED_BT_GATT_SUCCESS;
    } else if (event == GATT_CONNECTION_STATUS_EVT) {
        if (eventData->connection_status.connected) {
            isConnected = 1;
            isAdvertising = 0;
        } else {
            isConnected = 0;
            wStatus = wiced_bt_start_advertisements(BTM_BLE_ADVERT_UNDIRECTED_HIGH, 0, NULL);
            CY_ASSERT(wStatus == WICED_SUCCESS);
            isAdvertising = 1;
        }
        printf("%d\r\n", WICED_BT_GATT_SUCCESS);
        return WICED_BT_GATT_SUCCESS;
    } else {
        printf("%d\r\n", WICED_BT_GATT_ERROR);
        return WICED_BT_GATT_ERROR;
    }
}

wiced_bt_gatt_status_t BLE_GattRead(uint16_t connId, wiced_bt_gatt_opcode_t opcode, wiced_bt_gatt_read_t* req, uint16_t len) {
    wiced_bt_gatt_status_t wStatus;

    printf("BLE_GattRead(len=%d)\r\n", len);

    if (len < 2) {
        __BKPT(0);
        return WICED_BT_GATT_INVALID_ATTR_LEN;
    }

    if (req->offset != 0) {
        __BKPT(0);
        return WICED_BT_GATT_INVALID_OFFSET;
    }

    if (req->handle == HDLC_RADARCONFIG_HOLDTIME_VALUE) {
        wStatus = wiced_bt_gatt_server_send_read_handle_rsp(connId, opcode, 2, (uint8_t*)&desiredHoldTime, NULL);
        CY_ASSERT(wStatus == WICED_BT_GATT_SUCCESS);
        return wStatus;
    } else if (req->handle == HDLC_RADARCONFIG_THRESHOLD_VALUE) {
        wStatus = wiced_bt_gatt_server_send_read_handle_rsp(connId, opcode, 2, (uint8_t*)&desiredThreshold, NULL);
        CY_ASSERT(wStatus == WICED_BT_GATT_SUCCESS);
        return wStatus;
    } else {
        // try read standard fields
        for (int i = 0; i < app_gatt_db_ext_attr_tbl_size; i++) {
            if (app_gatt_db_ext_attr_tbl[i].handle == req->handle) {
				uint8_t* data = app_gatt_db_ext_attr_tbl[i].p_data;
				uint16_t dataLen = app_gatt_db_ext_attr_tbl[i].cur_len;

				if (req->offset > dataLen) {
        			return WICED_BT_GATT_INVALID_OFFSET;
				}

				data += req->offset;
				dataLen -= req->offset;
				if (dataLen > len) {
					dataLen = len;
				}

        		wStatus = wiced_bt_gatt_server_send_read_handle_rsp(connId, opcode, dataLen, data, NULL);
        		CY_ASSERT(wStatus == WICED_BT_GATT_SUCCESS);

		        return wStatus;
            }
        }
        return WICED_BT_GATT_READ_NOT_PERMIT;
    }
}

wiced_bt_gatt_status_t BLE_GattWrite(uint16_t connId, wiced_bt_gatt_opcode_t opcode, wiced_bt_gatt_write_req_t* req) {
    wiced_bt_gatt_status_t wStatus;
    printf("BLE_GattWrite()\r\n");

    if (req->handle == HDLC_RADARCONFIG_THRESHOLD_VALUE) {
        if (req->val_len != 2) {
            return WICED_BT_GATT_INVALID_ATTR_LEN;
        }

        desiredThreshold = *((uint16_t*)req->p_val);
    } else if (req->handle == HDLC_RADARCONFIG_HOLDTIME_VALUE) {
        if (req->val_len != 2) {
            return WICED_BT_GATT_INVALID_ATTR_LEN;
        }

        desiredHoldTime = *((uint16_t*)req->p_val);
    } else {
        return WICED_BT_GATT_WRITE_NOT_PERMIT;
    }

    wStatus = wiced_bt_gatt_server_send_write_rsp(connId, opcode, req->handle);
    CY_ASSERT(wStatus == WICED_BT_GATT_SUCCESS);

    return WICED_BT_GATT_SUCCESS;
}

wiced_result_t BLE_ManagementCallback(wiced_bt_management_evt_t event, wiced_bt_management_evt_data_t* eventData) {
    printf("BLE_ManagementCallback(%d) => ", event);

    wiced_result_t result;

    if (event == BTM_ENABLED_EVT) {
        result = wiced_bt_gatt_register(BLE_GattCallback);
        CY_ASSERT(result == WICED_BT_SUCCESS);

        result = wiced_bt_gatt_db_init(gatt_database, gatt_database_len, NULL);
        CY_ASSERT(result == WICED_BT_SUCCESS);

        wiced_bt_set_pairable_mode(WICED_FALSE, WICED_FALSE);

        result = wiced_bt_ble_set_raw_advertisement_data(CY_BT_ADV_PACKET_DATA_SIZE, cy_bt_adv_packet_data);
        CY_ASSERT(result == WICED_BT_SUCCESS);

        BLE_StartAdvertising();

        printf("%d\r\n", WICED_BT_SUCCESS);
        return WICED_BT_SUCCESS;
    } else if (event == BTM_LOCAL_IDENTITY_KEYS_UPDATE_EVT) {
        printf("%d\r\n", WICED_BT_SUCCESS);
        return WICED_BT_SUCCESS;
    } else if (event == BTM_BLE_ADVERT_STATE_CHANGED_EVT) {
        if (eventData->ble_advert_state_changed == BTM_BLE_ADVERT_OFF) {
            isAdvertising = 0;
        } else {
            isAdvertising = 1;
        }
        printf("%d\r\n", WICED_BT_SUCCESS);
        return WICED_BT_SUCCESS;
    }

    printf("%d\r\n", WICED_BT_ERROR);
    return WICED_BT_ERROR;
}

void BLE_StartAdvertising() {
    wiced_result_t wStatus;
    wStatus = wiced_bt_start_advertisements(BTM_BLE_ADVERT_UNDIRECTED_HIGH, 0, NULL);
    CY_ASSERT(wStatus == WICED_SUCCESS);
}
