/* std lib includes */
#include <malloc.h>
#include <stdint.h>

/* FreeRTOS*/
#include <FreeRTOS.h>
#include <task.h>

/* Sensor */
#include "BGT60LTR11.h"

/* Cypress Libs */
#include "cybsp.h"
#include "cyhal.h"
#include "cy_retarget_io.h"
#include "cy_serial_flash_qspi.h"
#include "cycfg_qspi_memslot.h"

/* BLE */
#include "BLE.h"

#define FAN_SWITCH_HISTHERESIS_MS 10000

// magic number for checking that FRAM contains valid settings, otherwise it 
// is supposed that firmware was upgraded or device was never used yet.
#define FRAM_LAYOUT_VERSION 0x69f86c03

BGT60LTR11_Status BGT60LTR11_WriteReg(BGT60LTR11_Device* dev, uint8_t address, uint16_t data);
BGT60LTR11_Status BGT60LTR11_ReadReg(BGT60LTR11_Device* dev, uint8_t address, uint16_t* data);
BGT60LTR11_Status BGT60LTR11_ReadRegBurst(BGT60LTR11_Device* dev, uint8_t address, uint16_t* data, size_t dataSize);

static uint16_t actualThreshold;
uint16_t desiredThreshold;

static uint16_t actualHoldTime;
uint16_t desiredHoldTime;

static BGT60LTR11_Device radar;

static TaskHandle_t radarTask;
static StaticTask_t radarStaticTask;
static StackType_t radarTaskStack[512];

static TaskHandle_t ledTask;
static StaticTask_t ledStaticTask;
static StackType_t ledTaskStack[64];

static TaskHandle_t buttonTask;
static StaticTask_t buttonStaticTask;
static StackType_t buttonTaskStack[512];

static TaskHandle_t detectionTask;
static StaticTask_t detectionStaticTask;
static StackType_t detectionTaskStack[64];

void SaveSettingsToFram();

void RadarTask(void* dummy) {
    while (1) {
        if (actualHoldTime != desiredHoldTime || actualThreshold != desiredThreshold) {
            BGT60LTR11_SoftReset(&radar);

            BGT60LTR11_Configuration config;
            BGT60LTR11_LoadDefaultConfiguration(&config);

            config.detector.holdTime = desiredHoldTime;
            config.detector.threshold = desiredThreshold;

            BGT60LTR11_SetConfiguration(&radar, &config);

    		uint16_t regs[57];
    		BGT60LTR11_ReadRegBurst(&radar, 0, regs, sizeof(regs));

    		printf("============= REGDUMP =============\r\n");
    		for (int i = 0; i < 57; i++) {
    		    printf("reg%d = 0x%04x\r\n", i, regs[i]);
    		}
    		printf("===================================\r\n");

            BGT60LTR11_SwitchToAutonomousPulsedMode(&radar);

            actualHoldTime = config.detector.holdTime;
            actualThreshold = config.detector.threshold;

			SaveSettingsToFram();

            printf("Radar reconfigured. New settings: threshold=%d, holdtime=%d\r\n", actualThreshold, actualHoldTime);
        } else {
            vTaskDelay(100);
        }
    }
}

void LedTask(void* dumny) {
    while (1) {
        if (isConnected) {
            // GREEN
            cyhal_gpio_write(CYBSP_LED_RGB_RED, 1);
            cyhal_gpio_write(CYBSP_LED_RGB_GREEN, 0);
            cyhal_gpio_write(CYBSP_LED_RGB_BLUE, 1);
            vTaskDelay(pdMS_TO_TICKS(10));
        } else if (isAdvertising) {
            // BLUE
            cyhal_gpio_write(CYBSP_LED_RGB_RED, 1);
            cyhal_gpio_write(CYBSP_LED_RGB_GREEN, 1);
            cyhal_gpio_write(CYBSP_LED_RGB_BLUE, 0);
            vTaskDelay(pdMS_TO_TICKS(200));

            // YELLOW (GREEN 25% duty cycle)
            cyhal_gpio_write(CYBSP_LED_RGB_RED, 0);
            cyhal_gpio_write(CYBSP_LED_RGB_BLUE, 1);
            for (int i = 0; i < 50; i++) {
                cyhal_gpio_write(CYBSP_LED_RGB_GREEN, 0);
                vTaskDelay(pdMS_TO_TICKS(1));
                cyhal_gpio_write(CYBSP_LED_RGB_GREEN, 1);
                vTaskDelay(pdMS_TO_TICKS(3));
            }
        } else {
            // OFF
            cyhal_gpio_write(CYBSP_LED_RGB_RED, 1);
            cyhal_gpio_write(CYBSP_LED_RGB_GREEN, 1);
            cyhal_gpio_write(CYBSP_LED_RGB_BLUE, 1);
            vTaskDelay(pdMS_TO_TICKS(10));
        }
    }
}

void ButtonTask(void* dummy) {
    int prevState = cyhal_gpio_read(CYBSP_SW2);
    while (1) {
        int currState = cyhal_gpio_read(CYBSP_SW2);
        if (prevState == 1 && currState == 0) {
            if (!isConnected && !isAdvertising) {
				BLE_StartAdvertising();
                isAdvertising = 1;
            }
        }

        // debouncing delay
		if (prevState != currState) {
            vTaskDelay(pdMS_TO_TICKS(30));
		}

        prevState = currState;
        vTaskDelay(pdMS_TO_TICKS(3));
    }
}

void DetectionTask(void* dummy) {
	TickType_t lastMotion = 0;
    while (1) {
		int motionDetected = !cyhal_gpio_read(RADAR_TDET);
		TickType_t now = xTaskGetTickCount();

		if (motionDetected) {
			lastMotion = now;
			cyhal_gpio_write(PROFET_SHIELD_IN1, 1);
		} else if (now - lastMotion > pdMS_TO_TICKS(FAN_SWITCH_HISTHERESIS_MS)) {
			cyhal_gpio_write(PROFET_SHIELD_IN1, 0);
		}

        vTaskDelay(pdMS_TO_TICKS(1));
	}
}

void SaveSettingsToFram() {
    cy_rslt_t cyStatus;

	uint32_t framData[3] = {FRAM_LAYOUT_VERSION, actualThreshold, actualHoldTime};

	cyStatus = cy_serial_flash_qspi_write(0, sizeof(framData), (uint8_t*)framData);
    CY_ASSERT(cyStatus == CY_RSLT_SUCCESS);
}

int main(void) {
    cy_rslt_t cyStatus;
	BGT60LTR11_Status bStatus;

    cyStatus = cybsp_init();
    CY_ASSERT(cyStatus == CY_RSLT_SUCCESS);

    __enable_irq();

    cyStatus = cy_retarget_io_init(CYBSP_DEBUG_UART_TX, CYBSP_DEBUG_UART_RX, CY_RETARGET_IO_BAUDRATE);
    CY_ASSERT(cyStatus == CY_RSLT_SUCCESS);

	cyStatus = cy_serial_flash_qspi_init(smifMemConfigs[1], P11_6, P11_5, P11_4, P11_3, NC, NC, NC, NC, P11_7, P11_0, 50000000);
    CY_ASSERT(cyStatus == CY_RSLT_SUCCESS);

	uint32_t framData[3] = {};
	cyStatus = cy_serial_flash_qspi_read(0, sizeof(framData), (uint8_t*)framData);
    CY_ASSERT(cyStatus == CY_RSLT_SUCCESS);

	if (framData[0] == FRAM_LAYOUT_VERSION) {
		desiredThreshold = (uint16_t)framData[1];
		desiredHoldTime = (uint16_t)framData[2];
		printf("Recovered settings from FRAM. Threshold = %d, Holdtime = %d\r\n", desiredThreshold, desiredHoldTime);
	} else {
		// use defaults
		desiredThreshold = 400;
		desiredHoldTime = 16;
	}
	// set acutal to different value than desired. This will trigger reconfiguration in Radar task right after starting.
	actualThreshold = desiredThreshold - 1;
	actualHoldTime = desiredHoldTime - 1;

    bStatus = BGT60LTR11_Init(&radar, CY_SCB_SPI_SLAVE_SELECT0);
    CY_ASSERT(bStatus == BGT60LTR11_Status_Ok);

    radarTask = xTaskCreateStatic(RadarTask, "radar", 512, NULL, 0, radarTaskStack, &radarStaticTask);
    CY_ASSERT(radarTask != NULL);

    ledTask = xTaskCreateStatic(LedTask, "led", 64, NULL, 0, ledTaskStack, &ledStaticTask);
    CY_ASSERT(ledTask != NULL);

    buttonTask = xTaskCreateStatic(ButtonTask, "button", 512, NULL, 0, buttonTaskStack, &buttonStaticTask);
    CY_ASSERT(buttonTask != NULL);
	
    detectionTask = xTaskCreateStatic(DetectionTask, "button", 64, NULL, 0, detectionTaskStack, &detectionStaticTask);
    CY_ASSERT(detectionTask != NULL);

    BLE_Init();

    vTaskStartScheduler();

    while (1) {
        Cy_SysLib_Delay(250);
    }
}
